﻿Option Strict On
'Download: http://support.h02.ir/fwlink/?LinkId=1005796407

Friend Class frmMain

    Public Const PLUGINFOLDER As String = "Plugins\"
    Public Const PLUGINEXTENSION As String = "*.dll"

    Public Sub New()
        MyBase.New()
        Me.InitializeComponent()

        '-----

        Dim pluginpath As String = System.IO.Path.Combine(System.Windows.Forms.Application.StartupPath, PLUGINFOLDER)

        'Sample 1 ...
        Me.lstControls.DataSource = Me.LoadControls(pluginpath)
        Me.lstControls.DisplayMember = "Name"
        Me.lstControls.ValueMember = "Type"

        'Sample 2 ...
        Me.lstIPlugins.DataSource = Me.LoadIPlugins(pluginpath)
        Me.lstIPlugins.DisplayMember = "Name"
        Me.lstIPlugins.ValueMember = "Type"
    End Sub

#Region " Sample 1 <System.Windows.Forms.Control> "

    'Sample 1 ...
    Private Function LoadControls(ByVal path As String) As System.Collections.Generic.ICollection(Of ExternalType)
        Dim ret As New System.Collections.Generic.List(Of ExternalType)()
        Dim files() As String = System.IO.Directory.GetFiles(path, PLUGINEXTENSION, System.IO.SearchOption.TopDirectoryOnly)
        If ((files Is Nothing) OrElse (files.Length <= 0)) Then Return ret
        Dim evidence As System.Security.Policy.Evidence = System.Reflection.Assembly.GetExecutingAssembly().Evidence
        Dim basetype As Type = GetType(System.Windows.Forms.Control)
        '-----
        For Each file As String In files
            Try
                Dim asm As System.Reflection.Assembly = System.Reflection.Assembly.LoadFile(file, evidence)

                For Each type As Type In asm.GetTypes()
                    Try
                        If (type.IsAbstract OrElse type.IsArray OrElse type.IsEnum OrElse type.IsGenericType OrElse type.IsInterface) Then Continue For

                        If (basetype.IsAssignableFrom(type)) Then
                            ret.Add(New ExternalType(type.FullName, type))
                        End If
                    Catch
                    End Try
                Next
            Catch
            End Try
        Next
        Return ret
    End Function

    'Sample 1 ...
    Private Sub lstControls_DoubleClick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles lstControls.DoubleClick
        Try
            Dim type As ExternalType = TryCast(Me.lstControls.SelectedItem, ExternalType)
            If (type Is Nothing) Then Return

            Using obj As System.Windows.Forms.Control = type.CreateNew(Of System.Windows.Forms.Control)()
                If (obj Is Nothing) Then Return

                Using form As New System.Windows.Forms.Form()
                    form.Size = obj.Size
                    form.Text = "Test Control"

                    obj.Dock = System.Windows.Forms.DockStyle.Fill
                    form.Controls.Add(obj)

                    form.ShowDialog(Me)
                End Using
            End Using

        Catch ex As Exception
            Me.OnException(ex)
        End Try
    End Sub

#End Region

#Region " Sample 2 <IPlugin> "

    'Sample 2 ...
    Private Function LoadIPlugins(ByVal path As String) As System.Collections.Generic.ICollection(Of ExternalType)
        Dim ret As New System.Collections.Generic.List(Of ExternalType)()
        Dim files() As String = System.IO.Directory.GetFiles(path, PLUGINEXTENSION, System.IO.SearchOption.TopDirectoryOnly)
        If ((files Is Nothing) OrElse (files.Length <= 0)) Then Return ret
        Dim evidence As System.Security.Policy.Evidence = System.Reflection.Assembly.GetExecutingAssembly().Evidence
        Dim basetype As Type = GetType(IPlugin)
        '-----
        For Each file As String In files
            Try
                Dim asm As System.Reflection.Assembly = System.Reflection.Assembly.LoadFile(file, evidence)

                For Each type As Type In asm.GetTypes()
                    Try
                        If (type.IsAbstract OrElse type.IsArray OrElse type.IsEnum OrElse type.IsGenericType OrElse type.IsInterface) Then Continue For

                        If (basetype.IsAssignableFrom(type)) Then
                            Using Plugin As IPlugin = CType(System.Activator.CreateInstance(type), IPlugin)
                                ret.Add(New ExternalType(Plugin.Name, type))
                            End Using
                        End If
                    Catch
                    End Try
                Next
            Catch
            End Try
        Next
        Return ret
    End Function

    'Sample 2 ...
    Private Sub lstIPlugins_DoubleClick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles lstIPlugins.DoubleClick
        Try
            Dim type As ExternalType = TryCast(Me.lstIPlugins.SelectedItem, ExternalType)
            If (type Is Nothing) Then Return

            Dim strresult As String

            Using obj As IPlugin = type.CreateNew(Of IPlugin)()
                If (obj Is Nothing) Then Return

                Dim objresult As Object = obj.Run(Nothing)
                strresult = If(objresult Is Nothing, "(Nothing)", objresult.ToString())
            End Using

            System.Windows.Forms.MessageBox.Show(Me, "Input: (Nothing)" & vbCrLf & "Result: " & strresult, "IPlugin.Run")

        Catch ex As Exception
            Me.OnException(ex)
        End Try
    End Sub

#End Region

    Private Sub OnException(ByVal value As Exception)
        If (value IsNot Nothing) Then
            System.Windows.Forms.MessageBox.Show( _
                Me, _
                value.Message, _
                "Error", _
                System.Windows.Forms.MessageBoxButtons.OK, _
                System.Windows.Forms.MessageBoxIcon.Error)
        End If
    End Sub





    <STAThread()> Public Shared Sub Main()
        System.Windows.Forms.Application.EnableVisualStyles()
        System.Windows.Forms.Application.Run(New frmMain())
    End Sub

End Class
